from http.server import BaseHTTPRequestHandler, HTTPServer
import threading
import io
import time
import cv2
import numpy as np
from picamera2 import Picamera2
from rpi_ws281x import PixelStrip, Color

LED_COUNT = 16
LED_PIN = 18
LED_FREQ_HZ = 800000
LED_DMA = 10
LED_BRIGHTNESS = 255
LED_INVERT = False
LED_CHANNEL = 0

strip = PixelStrip(LED_COUNT, LED_PIN, LED_FREQ_HZ, LED_DMA, LED_INVERT,

LED_BRIGHTNESS, LED_CHANNEL)

strip.begin()

# Inicjalizacja kamery
picam2 = Picamera2()
picam2.configure(picam2.create_preview_configuration(main={"format": "RGB888", "size": (640, 480)}))
picam2.start()

# Ładowanie klasyfikatora Haar Cascade
# face_cascade = cv2.CascadeClassifier(cv2.data.haarcascades + "haarcascade_frontalface_default.xml")

recognizer = cv2.face.LBPHFaceRecognizer_create()
recognizer.read('trainer/trainer.yml')
# recognizer.load('trainer/trainer.yml')
face_cascade = cv2.CascadeClassifier('Cascades/haarcascade_frontalface_default.xml')
font = cv2.FONT_HERSHEY_SIMPLEX
#identyfikator rozpoznawanych osob
id = 0
# tu mozemy przypisac nazwy do poszczegolnych identyfikatorow: np. ==> Krzysiek: id=1,  itd.
names = ['Nieznany', 'Krzysiek', 'Rysiek']

frame_lock = threading.Lock()
output_frame = None

# Funkcja do przechwytywania i przetwarzania obrazu
def capture_frames():
    global output_frame, id, names, strip
    while True:
        frame = picam2.capture_array()
        gray = cv2.cvtColor(frame, cv2.COLOR_RGB2GRAY)

        # Wykrywanie twarzy
        faces = face_cascade.detectMultiScale(gray, scaleFactor=1.2, minNeighbors=5, minSize=(64, 48))

        if len(faces) == 0:
            for i in range(strip.numPixels()):
                strip.setPixelColor(i, Color(0, 0, 0))
            strip.show()

        # Rysowanie prostokątów wokół twarzy
        for (x, y, w, h) in faces:
            cv2.rectangle(frame, (x, y), (x+w, y+h), (0, 255, 0), 2)
            id, confidence = recognizer.predict(gray[y:y+h,x:x+w])
            # Sprawdzamy pewnosc naszego rozpoznania od 100 do 0 ==> "0" rozpoznanie idealne 
            if (confidence < 95):
                id_name = names[id]
                # print(names[id])
                confidence = "  {0}%".format(round(100 - confidence))
                for i in range(strip.numPixels()):
                    strip.setPixelColor(i, Color(255, 0, 0))
                strip.show()                
            else:
                id_name = names[0] #brak rozpoznania
                confidence = "  "
            
            cv2.putText(frame, str(id_name), (x+5,y-5), font, 1, (255,255,255), 2)
            cv2.putText(frame, str(confidence), (x+5,y+h-5), font, 1, (255,255,0), 1) 

        # Kodowanie do JPEG
        ret, jpeg = cv2.imencode('.jpg', frame)
        if ret:
            with frame_lock:
                output_frame = jpeg.tobytes()

# Serwer HTTP MJPEG
class MJPEGHandler(BaseHTTPRequestHandler):
    def do_GET(self):
        if self.path != '/':
            self.send_error(404)
            return

        self.send_response(200)
        self.send_header("Content-type", "multipart/x-mixed-replace; boundary=frame")
        self.end_headers()

        while True:
            with frame_lock:
                if output_frame is None:
                    continue
                frame = output_frame

            try:
                self.wfile.write(b"--frame\r\n")
                self.wfile.write(b"Content-Type: image/jpeg\r\n\r\n" + frame + b"\r\n")
                time.sleep(0.1)
            except Exception as e:
                print("Client disconnected:", e)
                break

# Start wątku kamery
threading.Thread(target=capture_frames, daemon=True).start()

# Uruchomienie serwera
def run_server():
    server_address = ('', 8000)
    httpd = HTTPServer(server_address, MJPEGHandler)
    print("Serwer działa na porcie 8000...")
    httpd.serve_forever()

run_server()
